/*
   plaympeg - Sample MPEG player using the SMPEG library
   Copyright (C) 1999 Loki Entertainment Software
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <string.h>

#include "smpeg.h"


#ifdef MPEG_EXPORT
#include "mpeg_export.h"
#endif


void usage(char *argv0)
{
#ifdef MPEG_EXPORT
    printf(
		"Usage: %s [--noaudio] [--novideo] [--double|-2] [--loop|-l] [--export|-e] file ...\n",
		argv0);
#else
    printf(
		"Usage: %s [--noaudio] [--novideo] [--double|-2] [--loop|-l] file ...\n",
		argv0);

#endif
}

int main(int argc, char *argv[])
{
    int use_audio, use_video;
    int doublesize;
    int loop_play;
    int i;
    SMPEG *mpeg;
    SMPEG_Info info;
    char *basefile;

    /* Initialize SDL */
    if ( SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0 ) {
        fprintf(stderr, "Couldn't init SDL: %s\n", SDL_GetError());
        exit(1);
    }

/* Initialize exporter */
#ifdef MPEG_EXPORT
	exporter_init();
#endif

    atexit(SDL_Quit);

    /* Get the command line options */
    use_audio = 1;
    use_video = 1;
    doublesize = 0;
    loop_play = 0;
    for ( i=1; argv[i] && (argv[i][0] == '-'); ++i ) {
        if ( strcmp(argv[i], "--noaudio") == 0 ) {
            use_audio = 0;
        } else
        if ( strcmp(argv[i], "--novideo") == 0 ) {
            use_video = 0;
        } else
        if ((strcmp(argv[i], "--double") == 0)||(strcmp(argv[i], "-2") == 0)) {
            doublesize = 1;
        } else
        if ((strcmp(argv[i], "--loop") == 0) || (strcmp(argv[i], "-l") == 0)) {
            loop_play = 1;
        } else
        if ((strcmp(argv[i], "--help") == 0) || (strcmp(argv[i], "-h") == 0)) {
            usage(argv[0]);
            exit(0);
        } else
#ifdef MPEG_EXPORT
        if ((strcmp(argv[i], "--export") == 0) || (strcmp(argv[i], "-e") == 0)) {
			export_data.export_it = 1;
        } else 
#endif
		{
            fprintf(stderr, "Warning: Unknown option: %s\n", argv[i]);
        }
    }

    /* Play the mpeg files! */
    for ( ; argv[i]; ++i ) {
        /* Create the MPEG stream */
        mpeg = SMPEG_new(argv[i], &info);
        if ( SMPEG_error(mpeg) ) {
            fprintf(stderr, "%s: %s\n", argv[i], SMPEG_error(mpeg));
            SMPEG_delete(mpeg);
            continue;
        }
        SMPEG_enableaudio(mpeg, use_audio);
        SMPEG_enablevideo(mpeg, use_video);

        /* Print information about the video */
        basefile = strrchr(argv[i], '/');
        if ( basefile ) {
            ++basefile;
        } else {
            basefile = argv[i];
        }
        if ( info.has_audio && info.has_video ) {
            printf("%s: MPEG system stream (audio/video)\n", basefile);
        } else if ( info.has_audio ) {
            printf("%s: MPEG audio stream\n", basefile);
        } else if ( info.has_video ) {
            printf("%s: MPEG video stream\n", basefile);
        }
        if ( info.has_video ) {
            printf("\tVideo %dx%d resolution\n", info.width, info.height);
        }

info.width += 2;

#ifdef MPEG_EXPORT
		if(export_data.export_it)
		{
   			printf("Exporting audio to audio.pcm.  Sorry.  SMpeg video is too buggy.\n");
//  			export_data.video_file = quicktime_open(export_data.video_path, 0, 1);
			export_data.audio_file = fopen(export_data.audio_path, "wb");
//    		quicktime_set_video(export_data.video_file, 
//				1, 
//				info.width, 
//				info.height, 
//				24, 
//				export_data.vid_compressor);
			export_data.w = info.width;
			export_data.h = info.height;
		}
#endif



        /* Set up video display if needed */
        if ( info.has_video && use_video ) {
            SDL_Surface *screen;

            if ( doublesize ) {
                SMPEG_double(mpeg, 1);
                info.width *= 2;
                info.height *= 2;
            }
            screen = SDL_SetVideoMode(info.width, info.height, 0, 0);
            SMPEG_setdisplay(mpeg, screen, NULL, NULL);
        }

        /* Set any special playback parameters */
        if ( loop_play ) {
            SMPEG_loop(mpeg, 1);
        }

        /* Play it, and wait for playback to complete */
        SMPEG_play(mpeg);
       while (!SDL_QuitRequested() && (SMPEG_status(mpeg) == SMPEG_PLAYING)) {
            SDL_Delay(1000/2);
        }
       SMPEG_delete(mpeg);
    }

#ifdef MPEG_EXPORT
  if(export_data.export_it)
  {
    exporter_end();
  }
#endif

    exit(0);
}
