#include "quicktime.h"
#include "jpeglib.h"

int quicktime_init_codec_mjpa(quicktime_video_map_t *vtrack)
{
	int i, field_height, bytes_per_line;
	vtrack->codecs.mjpa_codec.quality = 100;
	vtrack->codecs.mjpa_codec.use_float = 0;
	field_height = vtrack->codecs.mjpa_codec.field_height = vtrack->track->tkhd.track_height / 2 + 1;
	bytes_per_line = vtrack->track->tkhd.track_width * 3;
	vtrack->codecs.mjpa_codec.work_buffer = 
		malloc(bytes_per_line * field_height);
	vtrack->codecs.mjpa_codec.row_pointers = malloc(sizeof(unsigned char*) * field_height);
	for(i = 0; i < field_height; i++)
	{
		vtrack->codecs.mjpa_codec.row_pointers[i] = 
			&(vtrack->codecs.mjpa_codec.work_buffer[(int)(vtrack->track->tkhd.track_width * 3 * i)]);
	}
}

int quicktime_delete_codec_mjpa(quicktime_video_map_t *vtrack)
{
	free(vtrack->codecs.mjpa_codec.row_pointers);
	free(vtrack->codecs.mjpa_codec.work_buffer);
}

int quicktime_set_mjpa(quicktime_t *file, int quality, int use_float)
{
	int i;
	char *compressor;

	for(i = 0; i < file->total_vtracks; i++)
	{
		if(quicktime_match_32(quicktime_video_compressor(file, i), QUICKTIME_MJPA))
		{
			quicktime_mjpa_codec_t *codec = &(file->vtracks[i].codecs.mjpa_codec);
			codec->quality = quality;
			codec->use_float = use_float;
		}
	}
}

int mjpa_decode_field(quicktime_t *file, int track, unsigned char **row_pointers, int odd)
{
	quicktime_trak_t *trak = file->vtracks[track].track;
	int height = trak->tkhd.track_height;
	int width = trak->tkhd.track_width;
	struct jpeg_decompress_struct jpeg_decompress;
	struct jpeg_error_mgr jpeg_error;
	long color_channels, bytes, i, j, k;
	quicktime_mjpa_codec_t *codec = &(file->vtracks[track].codecs.mjpa_codec);
	int field_height = codec->field_height;
	JSAMPROW *jpeg_rows = (JSAMPROW*)codec->row_pointers;
	int result = 0;

	jpeg_decompress.err = jpeg_std_error(&jpeg_error);
	jpeg_create_decompress(&jpeg_decompress);

printf("mjpa_decode_field %d\n", quicktime_position(file));
	if(!file->vtracks[track].frames_cached)
	{
		jpeg_stdio_src(&jpeg_decompress, quicktime_get_fd(file));
	}
	else
	{
		jpeg_buffer_src(&jpeg_decompress, 
			file->vtracks[track].frame_cache[file->vtracks[track].current_position],
			quicktime_frame_size(file, file->vtracks[track].current_position, track));
	}
	jpeg_read_header(&jpeg_decompress, TRUE);
	jpeg_start_decompress(&jpeg_decompress);
	color_channels = jpeg_decompress.jpeg_color_space;

// read the image
	while(jpeg_decompress.output_scanline < jpeg_decompress.output_height && 
		jpeg_decompress.output_scanline < field_height)
	{
		jpeg_read_scanlines(&jpeg_decompress, 
			(JSAMPROW*)&jpeg_rows[jpeg_decompress.output_scanline], 
			jpeg_decompress.output_height - jpeg_decompress.output_scanline);
	}
	jpeg_finish_decompress(&jpeg_decompress);
	jpeg_destroy((j_common_ptr)&jpeg_decompress);

	for(i = (odd ? 1 : 0), j = 0; i < height; i += 2)
	{
		for(k = 0; k < width * 3; k++)
		{
			row_pointers[i][k] = jpeg_rows[j][k];
		}
	}

	return result;
}

int quicktime_decode_mjpa(quicktime_t *file, unsigned char **row_pointers, int track)
{
	int result = 0;
	int i;

	if(!file->vtracks[track].frames_cached)
	{
		quicktime_set_video_position(file, file->vtracks[track].current_position, track);
	}

// Decode even frames
	result = mjpa_decode_field(file, track, row_pointers, 0);

// Decode odd frames
	result = mjpa_decode_field(file, track, row_pointers, 1);

	return result;
}

int quicktime_encode_mjpa(quicktime_t *file, unsigned char **row_pointers, int track)
{
	int result = 0;
	return result;
}
